-- addon name

local a,_=...

-- create frames

local NCT=CreateFrame('frame',a)
local NCTPlayerEvents=CreateFrame('frame',a..'Player')
local NCTPetEvents=CreateFrame('frame',a..'Pet')
local NCTPanel=CreateFrame('frame',a..'Panel')

-- initialize variables

NoCombatText_SV={}
local notemp=true
local numobj=0
local posbox=-50
local temp={}
local vars={}
local defs={
	player={WOUND=false,HEAL=false,ENERGIZE=false,},
	pet={WOUND=false,HEAL=false,ENERGIZE=false,},
}
for k in next,CombatFeedbackText do
	defs.player[k]=false
	defs.pet[k]=false
end
local elist={}
for k in pairs(defs.player) do
	if k == 'WOUND' then tinsert(elist,'DAMAGE')
	else tinsert(elist,k) end
end
table.sort(elist)
local txt={
	WOUND='|cffffff00'..DAMAGE..'',
	HEAL='|cff00ff00'..VOICEMACRO_LABEL_HEALME1..'',
	ENERGIZE='|cff69ccf0'..COMBAT_TEXT_SHOW_ENERGIZE_TEXT..'',
}

-- following function from http://lua-users.org/wiki/CopyTable
-- this copies the values of a table into another table instead of creating a reference

local function tcopy(src)
	local copy
	if type(src) == 'table' then
		copy={}
		for k,v in next,src do
			copy[tcopy(k)]=tcopy(v)
		end
		setmetatable(copy,tcopy(getmetatable(src)))
	else
		copy=src
	end
	return copy
end

-- login setup block

NCT:RegisterEvent('PLAYER_LOGIN')
NCT:SetScript('OnEvent',function()
	if NoCombatText_SV.check then
		for k1,v1 in next,defs do
			if NoCombatText_SV[k1] == nil then
				NoCombatText_SV[k1]=defs[k1]
				vars[k1]=defs[k1]
			else
				vars[k1]=NoCombatText_SV[k1]
			end
			for k2 in next,v1 do
				if NoCombatText_SV[k1][k2] == nil then
					NoCombatText_SV[k1][k2]=defs[k1][k2]
					vars[k1][k2]=defs[k1][k2]
				elseif type(NoCombatText_SV[k1][k2]) ~= 'boolean' then
					NoCombatText_SV[k1][k2]=defs[k1][k2]
					vars[k1][k2]=defs[k1][k2]
				else
					vars[k1][k2]=NoCombatText_SV[k1][k2]
				end
			end
		end
		NoCombatText_SV=nil
		NoCombatText_SV=tcopy(vars)
		NoCombatText_SV.check=true
	else
		vars=tcopy(defs)
		NoCombatText_SV=nil
		NoCombatText_SV=tcopy(defs)
		NoCombatText_SV.check=true
	end
	NCT:Options()
	NCT:Register()
	NCT:UnregisterEvent('PLAYER_LOGIN')
end)

-- event filter

local function NCTFeedback(_,_,unit,...)
	local event,flag,_=...
	local playerflag,petflag
	if vars.player[flag] ~= nil then playerflag=vars.player[flag] else playerflag=true end
	if vars.pet[flag] ~= nil then petflag=vars.pet[flag] else petflag=true end
	if unit == PlayerFrame.unit then
		if vars.player[event] and playerflag then CombatFeedback_OnCombatEvent(PlayerFrame,...) end
	end
	if unit == PetFrame.unit then
		if vars.pet[event] and petflag then CombatFeedback_OnCombatEvent(PetFrame,...) end
	end
end

-- event registration block

function NCT:Register()
	local regplayer=false
	local regpet=false
	for _,v in next,vars.player do
		if v then regplayer=true break end
	end
	for _,v in next,vars.pet do
		if v then regpet=true break end
	end
	PlayerFrame:UnregisterEvent('UNIT_COMBAT')
	PetFrame:UnregisterEvent('UNIT_COMBAT')
	NCTPlayerEvents:SetScript('OnEvent',NCTFeedback)
	NCTPetEvents:SetScript('OnEvent',NCTFeedback)
	if regplayer then NCTPlayerEvents:RegisterUnitEvent('UNIT_COMBAT','player','vehicle')
	else NCTPlayerEvents:UnregisterEvent('UNIT_COMBAT') end
	if regpet then NCTPetEvents:RegisterUnitEvent('UNIT_COMBAT','pet','player')
	else NCTPetEvents:UnregisterEvent('UNIT_COMBAT') end
end

-- interface panel

function NCT:Create(obj,xloc,yloc,text)
	numobj=numobj+1
	local name=a..'Object'..format('%02i',numobj)
	if obj == 'b' then
		obj=CreateFrame('CheckButton',name,NCTPanel,'InterfaceOptionsCheckButtonTemplate')
		obj:SetPoint('TOPLEFT',NCTPanel,'TOPLEFT',xloc,yloc)
		_G[name..'Text']:SetText(''..strlower(text)..'')
	elseif obj == 't' then
		obj=NCTPanel:CreateFontString(name,'ARTWORK','GameFontNormalLarge')
		obj:SetPoint('TOPLEFT',NCTPanel,'TOPLEFT',xloc,yloc)
		obj:SetText(text)
	end
	return obj
end

function NCT:Options()
	NCTPanel.name=a
	InterfaceOptions_AddCategory(NCTPanel,a)
	NCTPlayer=NCT:Create('t',20,-20,''..PLAYER..' '..ENABLE..'')
	NCTPet=NCT:Create('t',200,-20,''..PET..' '..ENABLE..'')
	for _,v in ipairs(elist) do
		if CombatFeedbackText[v] ~= nil then
			_G['NCTPlayer'..v]=NCT:Create('b',20,posbox,''..CombatFeedbackText[v]..'')
			_G['NCTPet'..v]=NCT:Create('b',200,posbox,''..CombatFeedbackText[v]..'')
		else
			if v == 'DAMAGE' then v='WOUND' end
			_G['NCTPlayer'..v]=NCT:Create('b',20,posbox,''..txt[v]..'')
			_G['NCTPet'..v]=NCT:Create('b',200,posbox,''..txt[v]..'')
		end
		_G['NCTPlayer'..v]:SetScript('PostClick',function(self)vars.player[v]=self:GetChecked()NCT:Register()end)
		_G['NCTPet'..v]:SetScript('PostClick',function(self)vars.pet[v]=self:GetChecked()NCT:Register()end)
		posbox=posbox-25
	end
	NCTPanel.okay=function()
		NoCombatText_SV=nil
		NoCombatText_SV=tcopy(vars)
		NoCombatText_SV.check=true
		notemp=true
	end
	NCTPanel.cancel=function()
		vars=tcopy(temp)
		notemp=true
		NCT:Register()
	end
	NCTPanel.default=function()
		vars=tcopy(defs)
		NoCombatText_SV=nil
		NoCombatText_SV=tcopy(defs)
		NoCombatText_SV.check=true
		NCT:SetCheckBoxes()
		NCT:Register()
	end
	NCTPanel.refresh=function()
		NCT:SetCheckBoxes()
		if notemp then temp=tcopy(vars) notemp=false end
	end
	SLASH_NOCOMBATTEXT_1='/nocombattext'
	SLASH_NOCOMBATTEXT_2='/nct'
	SlashCmdList['NOCOMBATTEXT_']=function()InterfaceOptionsFrame_OpenToCategory(a)end
end

function NCT:SetCheckBoxes()
	for k,v in next,vars.player do
		_G['NCTPlayer'..k]:SetChecked(v)
	end
	for k,v in next,vars.pet do
		_G['NCTPet'..k]:SetChecked(v)
	end
end